/*
 *	Copyright 2012  Samsung Electronics Co., Ltd
 *
 *	Licensed under the Flora License, Version 1.1 (the "License");
 *	you may not use this file except in compliance with the License.
 *	You may obtain a copy of the License at
 *
 * 		http://floralicense.org/license/
 *
 *	Unless required by applicable law or agreed to in writing, software
 *	distributed under the License is distributed on an "AS IS" BASIS,
 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *	See the License for the specific language governing permissions and
 *	limitations under the License.
 */


#include "mgr-app-common-debug.h"
#include "mgr-app-common-error.h"
#include "mgr-app-common-util.h"
#include "mgr-app-view-main.h"
#include "mgr-app-view-common.h"
#include "mgr-app-string.h"
#include "mgr-app-widget.h"
#include "mgr-app-uigadget.h"

#include <stdbool.h>
#include <package-manager.h>
#include <pkgmgr-info.h>
#include <appsvc.h>
#include <app_manager.h>

#define WRT_PREFIX 			"wgt"
#define TPK_PREFIX			"tpk"
#define RPM_PREFIX			"rpm"

#define STR_ICON			"icon"

#define EDC_FILE			EDJDIR"/ug-setting-manage-applications-efl/ug-setting-manage-applications-efl.edj"

#define ELM_TEXT			"elm.text"
#define ITC_ITEM_STYLE		"dialogue/1text.1icon.2"
#define ITC_SEP_ITEM_STYLE	"dialogue/separator"
#define GENLIST_BLOCK_CNT	10

#define MAX_BUF_SIZE		32
#define MAX_APP_CNT			1024
#define ALPHA_CNT			26
#define DIGIT_CNT			10

#define STR_INSTALL			"install"
#define STR_UNINSTALL		"uninstall"
#define STR_START			"start"
#define STR_END				"end"
#define STR_OK				"ok"
#define STR_CANCEL			"cancel"
#define STR_ERROR			"error"
#define STR_FAIL			"fail"
#define STR_POPUP_OK		"OK"

static Evas_Object *_genlist = NULL;
static Eina_Bool _initialize = false;
static GList *_installed_list = NULL;
static int _installed_cnt = 0;
static Elm_Object_Item *_tabbar_item[VIEW_TYPE_MAX] = {NULL,};
static Elm_Object_Item *_navi_it = NULL;

static Elm_Genlist_Item_Class *itc = NULL;
static Elm_Genlist_Item_Class *itc_sep = NULL;

static void _free_resources(struct ug_data * ugd)
{
	MGR_APP_BEGIN();

	mgr_app_app_info_t *app_info = NULL;
	int i = 0;

	for (i = 0; i < _installed_cnt; i++ ) {
		app_info = (mgr_app_app_info_t *)g_list_nth_data(_installed_list, 0);
		if (NULL == app_info) {
			continue;
		}
		MGR_APP_MEM_FREE(app_info->pkg_label);
		MGR_APP_MEM_FREE(app_info->pkg_name);
		MGR_APP_MEM_FREE(app_info->pkg_type);
		MGR_APP_MEM_FREE(app_info->icon_path);
		MGR_APP_MEM_FREE(app_info->main_appid);
		_installed_list = g_list_remove(_installed_list, app_info);
	}

	elm_genlist_item_class_free(itc);
	elm_genlist_item_class_free(itc_sep);
}

static void _app_list_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();
	ret_if(data == NULL);
	struct ug_data *ugd = (struct ug_data*)data;

	mgr_app_doubletap_sensor_disable();

	_free_resources(ugd);

	mgrapp_set_view_type(VIEW_TYPE_NONE);

	elm_naviframe_item_pop(ugd->navi_bar);

	/* bg must delete before starting on_destroy */
	if (ugd && ugd->bg) {
		evas_object_del(ugd->bg);
		ugd->bg = NULL;
	}

	if (ugd && ugd->ug) {
		ug_destroy_me(ugd->ug);
		ugd->ug = NULL;
	}

	MGR_APP_END();
}

/* callback for 'deletion' */
static void _gl_del(void *data, Evas_Object *obj)
{
	MGR_APP_BEGIN();
#if 0
	ret_if(data == NULL);
	mgr_app_app_info_t *app_info = (mgr_app_app_info_t *)data;

	// TODO
	MGR_APP_MEM_FREE(app_info->pkg_name);
	MGR_APP_MEM_FREE(app_info->pkg_label);
	MGR_APP_MEM_FREE(app_info->pkg_type);
	MGR_APP_MEM_FREE(app_info->icon_path);
	MGR_APP_MEM_FREE(app_info->main_appid);
	//MGR_APP_MEM_FREE(app_info);
#endif

	MGR_APP_END();
}

/* callback for 'selection' */
static void _gl_sel(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	ret_if(data == NULL);
	ret_if(event_info == NULL);

	struct ug_data *ugd = (struct ug_data*)data;
	Elm_Object_Item *item = (Elm_Object_Item *)event_info;

	mgr_app_app_info_t *app_info = (mgr_app_app_info_t *)elm_object_item_data_get(item);
	ret_if(app_info == NULL);

	elm_genlist_item_selected_set(item, 0);

	/* create app info view */
	ugd->data = (void*)app_info;
	mgrapp_view_info_create(app_info->pkg_name, (void*)ugd);

	MGR_APP_END();
}

static char *_gl_label_get(void *data, Evas_Object *obj, const char *part)
{
	MGR_APP_BEGIN();

	retv_if(data == NULL, NULL);
	mgr_app_app_info_t *app_info = (mgr_app_app_info_t *)data;

	char *label 		= NULL;
	char *package_label = app_info->pkg_label;
	char *icon_path 	= app_info->icon_path;

	MGR_APP_DEBUG("package_label:[%s], icon_path:[%s]\n", package_label, icon_path);
	MGR_APP_DEBUG("part:%s\n", part);

	/* label for 'elm.text' part */
	if (package_label && 0 == strcmp(part, ELM_TEXT)) {
		MGR_APP_MEM_STRDUP(label, package_label);
	} else {
		MGR_APP_MEM_STRDUP(label, MGRAPP_STR_ERROR);
	}

	MGR_APP_END();
	return label;
}

static Evas_Object *_gl_icon_get(void *data, Evas_Object *obj, const char *part)
{
	MGR_APP_BEGIN();

	retv_if(data == NULL, NULL);
	mgr_app_app_info_t *app_info = (mgr_app_app_info_t *)data;

	retv_if(app_info->icon_path == NULL, NULL);
	char *icon_path = NULL;
	icon_path = app_info->icon_path;

	Evas_Object *icon = NULL;

	MGR_APP_DEBUG("icon_path:[%s]\n", icon_path);
	MGR_APP_DEBUG("part:%s\n", part);

	/* icon for 'elm.icon' part */
	icon = elm_icon_add(obj);
	retv_if(icon == NULL, NULL);
	evas_object_size_hint_weight_set(icon, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(icon, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_aspect_set(icon, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);

	elm_image_file_set(icon, icon_path, STR_ICON);
	evas_object_show(icon);

	MGR_APP_END();
	return icon;
}

static gint _app_list_qsort_cb(gconstpointer a, gconstpointer b)
{
	MGR_APP_BEGIN();

	retv_if(NULL == a, 0);
	retv_if(NULL == b, 0);

	mgr_app_app_info_t *app_info1 = (mgr_app_app_info_t *)a;
	mgr_app_app_info_t *app_info2 = (mgr_app_app_info_t *)b;
	char *pkg_label1 = NULL;
	char *pkg_label2 = NULL;
	int len = 0;
	int r = 0;

	pkg_label1 = app_info1->pkg_label;
	pkg_label2 = app_info2->pkg_label;
	retv_if(NULL == pkg_label1, 0);
	retv_if(NULL == pkg_label2, 0);

	MGR_APP_DEBUG("pkg_label1: %s", pkg_label1);
	MGR_APP_DEBUG("pkg_label2: %s", pkg_label2);

	len = (strlen(pkg_label1) >= strlen(pkg_label2)) ? strlen(pkg_label1) : strlen(pkg_label2);
	r = strncasecmp(pkg_label1, pkg_label2, len);

	MGR_APP_END();
	return r;
}

static void _get_appinfo_from_pkgmgr(const pkgmgrinfo_pkginfo_h handler, mgr_app_app_info_t *app_info)
{
	MGR_APP_BEGIN();

	ret_if(NULL == handler || NULL == app_info);

	int r = 0;
	char *pkg_name = NULL;
	char *icon_path = NULL;
	char *main_appid = NULL;
	char *pkg_label = NULL;
	pkgmgrinfo_appinfo_h appinfo_h = NULL;

	r = pkgmgrinfo_pkginfo_get_pkgid(handler, &pkg_name);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_pkginfo_get_pkgid is failed %d", r);
		goto end;
	}
	MGR_APP_MEM_STRDUP(app_info->pkg_name, pkg_name);
	MGR_APP_DEBUG("pkg_name: %s", pkg_name);

	/* get label/icon from mainapp */
	r = pkgmgrinfo_pkginfo_get_mainappid(handler, &main_appid);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_pkginfo_get_mainappid is failed %d", r);
		goto end;
	}
	MGR_APP_DEBUG("main_appid: %s", main_appid);
	MGR_APP_MEM_STRDUP(app_info->main_appid, main_appid);

	r = pkgmgrinfo_appinfo_get_appinfo(main_appid, &appinfo_h);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_appinfo_get_appinfo is failed %d", r);
		goto end;
	}

	r = pkgmgrinfo_appinfo_is_nodisplay(appinfo_h, &app_info->nodisplay);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_appinfo_is_nodisplay failed. %d", r);
		goto end;
	}

	r = pkgmgrinfo_appinfo_get_icon(appinfo_h, &icon_path);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_appinfo_get_icon is failed %d", r);
		goto end;
	}
	MGR_APP_MEM_STRDUP(app_info->icon_path, icon_path);
	if (app_info->nodisplay) {
		mgr_app_set_default_icon(&app_info->icon_path, DEFAULT_ICON_SERVICE);
	} else {
		mgr_app_set_default_icon(&app_info->icon_path, DEFAULT_ICON_APP);
	}
	MGR_APP_DEBUG("icon_path: %s", icon_path);

	r = pkgmgrinfo_pkginfo_get_label(handler, &pkg_label);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgrinfo_pkginfo_get_label is failed %d", r);
		goto end;
	}
	MGR_APP_MEM_STRDUP(app_info->pkg_label, pkg_label);
	MGR_APP_DEBUG("pkg_label: %s", pkg_label);

end:
	if (appinfo_h) {
		pkgmgrinfo_appinfo_destroy_appinfo(appinfo_h);
	}

	MGR_APP_END();
}

static void _notify_by_uninstall(void *data)
{
	MGR_APP_BEGIN();

	ret_if(data == NULL);
	struct ug_data *ugd = (struct ug_data*)data;

	Evas_Object *noti = elm_notify_add(ugd->navi_bar);
	elm_notify_align_set(noti, ELM_NOTIFY_ALIGN_FILL, 1.0);

	Evas_Object *layout = elm_layout_add(noti);
	elm_layout_theme_set(layout, "standard", "selectioninfo", "center_text");

	elm_object_content_set(noti, layout);
	elm_notify_timeout_set(noti, 2);
	edje_object_part_text_set(_EDJ(layout), ELM_TEXT, MGRAPP_STR_UNINSTALLED);
	evas_object_show(noti);

	MGR_APP_END();
}

static void mgrapp_view_list_update_by_install(const char *pkg_type, const char *pkg_name, void *data)
{
	MGR_APP_BEGIN();

	ret_if(pkg_name == NULL || data == NULL);

	struct ug_data *ugd = (struct ug_data*)data;
	pkgmgr_pkginfo_h pkg_handle = NULL;
	mgr_app_app_info_t *app_info = NULL;
	Elm_Object_Item *item = NULL;
	int r = 0;

	_initialize = EINA_FALSE;

	r = pkgmgr_pkginfo_get_pkginfo(pkg_name, &pkg_handle);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("pkgmgr_pkginfo_get_pkginfo is failed. %d", r);
		goto end;
	}

	app_info = (mgr_app_app_info_t*)malloc(sizeof(mgr_app_app_info_t));
	if (NULL == app_info) {
		goto end;
	}
	memset(app_info, 0, sizeof(mgr_app_app_info_t));

	_get_appinfo_from_pkgmgr(pkg_handle, app_info);

	item = elm_genlist_item_append(_genlist, itc, (void*)app_info, NULL, ELM_GENLIST_ITEM_NONE, _gl_sel, (void*)ugd);
	if (item == NULL) {
		MGR_APP_DEBUG_ERR("It failed to append new item in gentlist");
	}

	_installed_cnt++;

end:
	if (pkg_handle) {
		pkgmgr_pkginfo_destroy_pkginfo(pkg_handle);
	}

	MGR_APP_END();
}

void mgrapp_view_list_update_by_uninstall(const char *pkg_type, const char *pkg_name, void *data)
{
	MGR_APP_BEGIN();

	ret_if(pkg_name == NULL);

	mgr_app_app_info_t *app_info = NULL;
	Elm_Object_Item *item = NULL;
	int i = 0;

	/* header separator */
	item = elm_genlist_first_item_get(_genlist);
	ret_if(item == NULL);
	item = elm_genlist_item_next_get(item);
	ret_if(item == NULL);


	for (i = 0; i < _installed_cnt; i++) {

		app_info = (mgr_app_app_info_t *)elm_object_item_data_get(item);
		ret_if(app_info == NULL);

		if (app_info->pkg_name && 0 == strcmp(app_info->pkg_name, pkg_name)) {
			_installed_list = g_list_remove(_installed_list, app_info);
			elm_object_item_del(item);
			item = NULL;

			MGR_APP_DEBUG("deleted");
			_installed_cnt--;
		}
		item = elm_genlist_item_next_get(item);
		ret_if(item == NULL);
	}

	elm_genlist_realized_items_update(_genlist);

	MGR_APP_END();
}

static void _response_cb(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	if (data) {
		evas_object_del(data);
		data = NULL;
	}

	MGR_APP_END();
}

int mgrapp_view_listen_cb(int req_id, const char *pkg_type, const char *pkg_name, const char *key, const char *val, const void *pmsg, void *data)
{
	MGR_APP_BEGIN();

	MGR_APP_DEBUG("key:%s val:%s", key, val);
	MGR_APP_DEBUG("pkg_type:%s pkg_name:%s", pkg_type, pkg_name);
	retv_if(data == NULL, -1);
	struct ug_data *ugd = (struct ug_data*)data;

	if (!strncmp(key, STR_START, strlen(key)) && !strncmp(val, STR_UNINSTALL, strlen(val))) {
		ugd->uninstall = EINA_TRUE;
	} else if (!strncmp(key, STR_START, strlen(key)) && !strncmp(val, STR_INSTALL, strlen(val))) {
		ugd->uninstall = EINA_FALSE;
	} else if (!strncmp(key, STR_ERROR, strlen(key))) {
	} else if (!strncmp(key, STR_END, strlen(key)) && !strncmp(val, STR_FAIL, strlen(val))) {
		mgrapp_destroy_popup_progressbar();
		mgrapp_create_popup_text_1button(ugd->win_main, MGRAPP_STR_FAILED, STR_POPUP_OK, _response_cb);
	} else if (!strncmp(key, STR_END, strlen(key)) && !strncmp(val, STR_OK, strlen(val))) {
		if (ugd->uninstall) {
			mgrapp_destroy_popup_progressbar();
			mgrapp_create_popup_text_1button(ugd->win_main, MGRAPP_STR_SUCCESS, STR_POPUP_OK, _response_cb);
			mgrapp_view_list_update_by_uninstall(pkg_type, pkg_name, (void*)ugd);
		} else {
			mgrapp_view_list_update_by_install(pkg_type, pkg_name, (void*)ugd);
		}

		/* check the info view was opened */
		if (VIEW_TYPE_APP_INFO == mgrapp_get_view_type()) {
			if (ugd->uninstall) {
				mgrapp_view_info_destroy(ugd);
				_notify_by_uninstall((void*)ugd);
				ugd->uninstall = EINA_FALSE;
			}
		}
	} else if (!strncmp(key, STR_END, strlen(key)) && !strncmp(val, STR_CANCEL, strlen(val))) {
	} else {
	}

	MGR_APP_END();
	return 0;
}

static void _gl_realized(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	ret_if(obj == NULL);
	ret_if(event_info == NULL);

	if (event_info == elm_genlist_item_next_get(elm_genlist_first_item_get(obj))) { // first is a separator
		elm_object_item_signal_emit(event_info, "elm,state,top", "");
	} else if (event_info == elm_genlist_item_prev_get(elm_genlist_last_item_get(obj))) { // last is a separator
		elm_object_item_signal_emit(event_info, "elm,state,bottom", "");
	} else {
		elm_object_item_signal_emit(event_info, "elm,state,center", "");
	}

	MGR_APP_END();
}

static int _pkgmgr_iter_fn(pkgmgr_pkginfo_h handle, void *data)
{
	MGR_APP_BEGIN();

	retv_if(NULL == handle, 0);

	mgr_app_app_info_t *app_info = NULL;

	app_info = (mgr_app_app_info_t*)malloc(sizeof(mgr_app_app_info_t));
	retv_if(app_info == NULL, -1);
	memset(app_info, 0, sizeof(mgr_app_app_info_t));

	_get_appinfo_from_pkgmgr(handle, app_info);

	_installed_list = g_list_append(_installed_list, (gpointer)app_info);
	_installed_cnt++;
	MGR_APP_DEBUG("installed cnt %d", _installed_cnt);

	MGR_APP_END();
	return 0;
}

static void _create_genlist_app_list(seg_type_t seg, void *data)
{
	MGR_APP_BEGIN();

	ret_if(data == NULL);
	struct ug_data	*ugd = (struct ug_data*)data;
	Evas_Object		*genlist 	= NULL;
	Elm_Object_Item	*item_sep	= NULL;
	Elm_Object_Item *item = NULL;

	mgr_app_app_info_t *app_info = NULL;

	int r = PKGMGR_R_ERROR;
	int i = 0;
	bool is_running = false;

	/* set the genlist item style */
	itc = elm_genlist_item_class_new();
	ret_if(itc == NULL);
	itc_sep = elm_genlist_item_class_new();
	ret_if(itc_sep == NULL);

	itc->item_style 	= ITC_ITEM_STYLE;
	itc->func.text_get 	= _gl_label_get;
	itc->func.content_get 	= _gl_icon_get;
	itc->func.state_get = NULL;
	itc->func.del 		= _gl_del;

	itc_sep->item_style		= ITC_SEP_ITEM_STYLE;
	itc_sep->func.text_get 	= NULL;
	itc_sep->func.content_get= NULL;
	itc_sep->func.state_get	= NULL;
	itc_sep->func.del		= NULL;

	/* add a genlist */
	genlist = elm_genlist_add(ugd->navi_bar);
	ret_if(genlist == NULL);
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);
	elm_genlist_block_count_set(genlist, GENLIST_BLOCK_CNT);
	elm_scroller_policy_set(genlist, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);
	evas_object_show(genlist);
	evas_object_smart_callback_add(genlist, "realized", _gl_realized, NULL);

	_genlist = genlist;

	/* header border */
	item_sep = elm_genlist_item_append(genlist, itc_sep, NULL, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (item_sep)
		elm_genlist_item_select_mode_set(item_sep, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);

	for (i = 0; i < _installed_cnt; i++) {

		app_info = (mgr_app_app_info_t *)g_list_nth_data(_installed_list, i);
		if (NULL == app_info) {
			MGR_APP_DEBUG_ERR("app_info is a null");
			continue;
		}
		MGR_APP_DEBUG("genlist %d app_info->pkg_label: %s", i, app_info->pkg_label);

		if (LIST_SEG_TYPE_RUNNING == seg
				&& APP_MANAGER_ERROR_NONE == app_manager_is_running(app_info->main_appid, &is_running)
				&& false == is_running) {
			continue;
		}

		item = elm_genlist_item_append(genlist, itc, (void*)app_info, NULL, ELM_GENLIST_ITEM_NONE, _gl_sel, (void*)ugd);
		if (item == NULL) {
			MGR_APP_DEBUG_ERR("It failed to append new item in gentlist");
		}
	}

	/* footer border */
	item_sep = elm_genlist_item_append(genlist, itc_sep, NULL, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (item_sep)
		elm_genlist_item_select_mode_set(item_sep, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);

	elm_object_item_part_content_set(_navi_it, "elm.swallow.content", genlist);

	/* register pkg-mgr listener */
	ret_if(ugd->pc_listen == NULL);
	r = pkgmgr_client_listen_status(ugd->pc_listen, mgrapp_view_listen_cb, (void*)ugd);
	if (r < 0) {
		MGR_APP_DEBUG_ERR("status listen failed!\n");
		return ;
	}

	MGR_APP_END();
}

static void _doubletap_sensor_cb(void *data)
{
	MGR_APP_BEGIN();

	ret_if(NULL == data);

	Evas_Object *genlist = (Evas_Object*)data;
	Elm_Object_Item *item = NULL;

	item = elm_genlist_first_item_get(genlist);
	ret_if(NULL == item);

	elm_genlist_item_show(item, ELM_GENLIST_ITEM_SCROLLTO_TOP);

	MGR_APP_END();
}

static void _create_genlist_app_cb(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	ret_if(NULL == data);

	_create_genlist_app_list(LIST_SEG_TYPE_APPLICATION, data);

	MGR_APP_END();
}

static void _create_genlist_all_cb(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	ret_if(NULL == data);

	_create_genlist_app_list(LIST_SEG_TYPE_ALL, data);

	MGR_APP_END();
}

static void _create_genlist_running_cb(void *data, Evas_Object *obj, void *event_info)
{
	MGR_APP_BEGIN();

	ret_if(NULL == data);

	_create_genlist_app_list(LIST_SEG_TYPE_RUNNING, data);

	MGR_APP_END();
}

void mgrapp_view_list_create(void *data)
{
	MGR_APP_BEGIN();

	ret_if(data == NULL);

	struct ug_data *ugd = (struct ug_data *)data;
	Evas_Object *back_btn = NULL;
	Evas_Object *tabbar = NULL;
	Elm_Object_Item *navi_it = NULL;
	int r = 0;

	mgrapp_set_view_type(VIEW_TYPE_APP_LIST);

	Evas_Object *nocontents = NULL;
	nocontents = elm_layout_add(ugd->navi_bar);
	ret_if(NULL == nocontents);
	elm_layout_theme_set(nocontents, "layout", "nocontents", "full");

	/* back button */
	back_btn = elm_button_add(ugd->navi_bar);
	ret_if(NULL == back_btn);
	elm_object_style_set(back_btn, "naviframe/end_btn/default");
	evas_object_smart_callback_add(back_btn, "clicked", _app_list_back_cb, (void*)ugd);

	_navi_it = navi_it = elm_naviframe_item_push(ugd->navi_bar, MGRAPP_STR_MANAGE_APPLICATIONS, back_btn, NULL, nocontents, "tabbar");

	r = pkgmgr_pkginfo_get_list(_pkgmgr_iter_fn, data);
	if (PKGMGR_R_OK != r) {
		MGR_APP_DEBUG_ERR("pkgmgr_get_pkg_list failed. %d", r);
	}

	/* sorting by pkg_label */
	_installed_list = g_list_sort(_installed_list, _app_list_qsort_cb);

	/* create toolbar */
	tabbar = elm_toolbar_add(ugd->navi_bar);
	ret_if(NULL == tabbar);
	elm_toolbar_shrink_mode_set(tabbar, ELM_TOOLBAR_SHRINK_EXPAND);
	elm_object_style_set(tabbar, "tabbar");
	elm_object_item_part_content_set(navi_it, "tabbar", tabbar);

	_tabbar_item[LIST_SEG_TYPE_APPLICATION] = elm_toolbar_item_append(tabbar, NULL, MGRAPP_STR_APPLICATION, _create_genlist_app_cb, (void*)ugd);
	_tabbar_item[LIST_SEG_TYPE_RUNNING] = elm_toolbar_item_append(tabbar, NULL, MGRAPP_STR_RUNNING, _create_genlist_running_cb, (void*)ugd);
	_tabbar_item[LIST_SEG_TYPE_ALL] = elm_toolbar_item_append(tabbar, NULL, MGRAPP_STR_ALL, _create_genlist_all_cb, (void*)ugd);
	elm_toolbar_item_selected_set(_tabbar_item[LIST_SEG_TYPE_APPLICATION], EINA_TRUE);

	mgr_app_doubletap_sensor_set_cb(_doubletap_sensor_cb, _genlist);
	mgr_app_doubletap_sensor_enable();

	MGR_APP_END();
}

void mgrapp_view_list_update(void *data)
{
	MGR_APP_BEGIN();

	if (_genlist) {
		mgr_app_doubletap_sensor_set_cb(_doubletap_sensor_cb, _genlist);
		mgr_app_doubletap_sensor_enable();
	}

	MGR_APP_END();
}
